# -*- coding: utf-8 -*-
#
# nti.identifiers documentation build configuration file, created by
# sphinx-quickstart on Thu Jun  8 07:02:25 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))
import os
import sys
import pkg_resources
sys.path.append(os.path.abspath('../src'))
rqmt = pkg_resources.require('nti.mailer')[0]

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'
# 1.8 was the last version that runs on Python 2; 2.0+ requires Python 3.
# `autodoc_default_options` was new in 1.8
needs_sphinx = "1.8"

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.coverage',
    'sphinx.ext.intersphinx',
    'sphinx.ext.extlinks',
    'sphinx.ext.viewcode',

    # Third-party
    'repoze.sphinx.autointerface',
    'sphinxcontrib.programoutput',

    # Ours

]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'nti.mailer'
copyright = u'2017, NextThought'
author = u'NextThought'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = '%s.%s' % tuple(map(int, rqmt.version.split('.')[:2]))
# The full version, including alpha/beta/rc tags.
release = rqmt.version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']


# -- Options for HTMLHelp output ------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'ntimailerdoc'


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'ntimailer.tex', u'nti.mailer Documentation',
     u'NextThought', 'manual'),
]


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'ntimailer', u'nti.identifiers Documentation',
     [author], 1)
]


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'ntimailer', u'nti.mailer Documentation',
     author, 'ntimailer', 'One line description of project.',
     'Miscellaneous'),
]




# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {
    "http://docs.pylonsproject.org/projects/pyramid/en/latest/": None,
    #'https://ntiwref.readthedocs.io/en/latest': None,
    'http://docs.python.org/': None,
    'http://ntischema.readthedocs.io/en/latest/': None,
    'http://ntizodb.readthedocs.io/en/latest/': None,
    'http://persistent.readthedocs.io/en/latest': None,
    'http://zodb.readthedocs.io/en/latest': None,

    'http://zopecomponent.readthedocs.io/en/latest': None,
    'http://zopecontainer.readthedocs.io/en/latest': None,
    'http://zopedatetime.readthedocs.io/en/latest': None,
    'http://zopedublincore.readthedocs.io/en/latest': None,
    'http://zopeevent.readthedocs.io/en/latest': None,
    'http://zopehookable.readthedocs.io/en/latest': None,
    'http://zopeinterface.readthedocs.io/en/latest': None,
    'http://zopeintid.readthedocs.io/en/latest/': None,
    'http://zopemimetype.readthedocs.io/en/latest/': None,
    'http://zopeproxy.readthedocs.io/en/latest': None,
    'http://zopeschema.readthedocs.io/en/latest/': None,
    'https://zopesecurity.readthedocs.io/en/latest/': None,
    'http://zopelifecycleevent.readthedocs.io/en/latest/': None,

    'https://docs.pylonsproject.org/projects/pyramid_mailer/en/latest/': None,
    'https://repozesendmail.readthedocs.io/en/latest/': None,
    'https://boto3.amazonaws.com/v1/documentation/api/latest/': None,
}


extlinks = {
    'issue': ('https://github.com/NextThought/nti.mailer/issues/%s',
              'issue #'),
    'pr': ('https://github.com/NextThought/nti.mailer/pull/%s',
           'pull request #')}


# Sphinx 1.8+ prefers this to `autodoc_default_flags`. It's documented that
# either True or None mean the same thing as just setting the flag, but
# only None works in 1.8 (True works in 2.0)
autodoc_default_options = {
    'members': None,
    'show-inheritance': None,
}

autoclass_content = 'both'
autodoc_member_order = 'bysource'
